
#include <math.h>
#include <stdio.h>
#include <string.h>
#include <sys/stat.h>

#include "off.h"

#define ABS( _x )     ((_x) < 0.00 ? (-(_x)) : (_x))
#define MAX( _x, _y ) ((_x) < (_y) ? (_y) : (_x))
#define MIN( _x, _y ) ((_x) < (_y) ? (_x) : (_y))

typedef struct _Vertex {
    float x, y, z;
} Vertex;

typedef Vertex Vector;

typedef struct _Edge {
    int e;
} Edge;

typedef struct _Facet {
    int count;
    Edge *edges;
} Facet;


main( argc, argv )
int argc;
char *argv[];
{
    OFFProperty *newprop;

    char *off_geomfile = NULL;

    char *temp, *rc;

    struct stat s_buf;

    char  *ptr;
    long  *lptr;
    float *fptr;
    short *sptr;

    long nvertices, nfacets, nedges;

    Vertex *vertices;
    Facet  *facets;
    Vertex max, min;
    float  max_val, min_val, floor_min, ceili_max;
    float  scale;
    float  interval, interval_start;

    int  i, j, k;

    if( argc != 2 ) {
        fprintf(stderr, "usage: %s off_filename\n", argv[0] );
        exit(-1);
    }
    
    off_geomfile = (char *) malloc( strlen( argv[1] ) + 1 );
    strcpy( off_geomfile, argv[1] );

    if( stat( off_geomfile, &s_buf ) == -1 ) {
        fprintf( stderr, "File \"%s\" does not exist\n", off_geomfile );
        exit(-1);
    }

    newprop = (OFFProperty *) malloc(sizeof(OFFProperty));
    newprop->PropType = OFF_INDEXED_POLY_DATA;
    newprop->NextProp = NULL;
    newprop->PropData = NULL;
    newprop->PropFileName[0] = '\0';
    newprop->DataFormat[0] = 'f';
    newprop->DataFormat[1] = 'f';
    newprop->DataFormat[2] = 'f';
    newprop->DataFormat[3] = '\0';
    newprop->PropCount = 0;
    newprop->PropName[0] = '\0';
    if( OFFReadIndexedPoly(newprop, off_geomfile) != 0 ) {
        fprintf( stderr, "Could not read file  \"%s\" \n", off_geomfile );
        exit(-1);
    }

    ptr = newprop->PropData;

    lptr = (long *) ptr;
    nvertices = *lptr++;
    nfacets   = *lptr++;
    nedges    = *lptr++;
    ptr = (char *) lptr;

    vertices = (Vertex *) malloc ( nvertices * sizeof( Vertex ) );
    facets   = (Facet *) malloc ( nfacets * sizeof( Facet ) );

    fptr = (float *) ptr;
    for(i = 0; i < nvertices; i++) {
        vertices[i].x = *fptr++;
        vertices[i].y = *fptr++;
        vertices[i].z = *fptr++;
    }
    ptr = (char *) fptr;

    sptr = (short *) ptr;
    for(j = 0; j < nfacets; j++) {
        facets[j].count = *sptr++;
        facets[j].edges = (Edge *) malloc ( facets[j].count * sizeof( Edge ) );
    }
    for(j = 0; j < nfacets; j++)
        for(i = 0; i < facets[j].count; i++)
            facets[j].edges[i].e = *sptr++;
    ptr = (char *) sptr;
/*
    Just define all points of all polygons in clockwise order.

           c = ((x3*((z1*y2)-(y1*z2))+
                (y3*((x1*z2)-(z1*x2))+
                (z3*((y1*x2)-(x1*y2)));

    x1,y1,z1, x2,y2,z2, x3,y3,z3 = the first three points of the
    polygon.

    If c is positive the polygon is visible. If c is negative the
    polygon is invisible (or the other way).
*/
    for(j = 0; j < nfacets; j++) {
if( ((vertices[facets[j].edges[2].e-1].x*((vertices[facets[j].edges[0].e-1].z*vertices[facets[j].edges[1].e-1].y)-(vertices[facets[j].edges[0].e-1].y*vertices[facets[j].edges[1].e-1].z)))+
     (vertices[facets[j].edges[2].e-1].y*((vertices[facets[j].edges[0].e-1].x*vertices[facets[j].edges[1].e-1].z)-(vertices[facets[j].edges[0].e-1].z*vertices[facets[j].edges[1].e-1].x)))+
     (vertices[facets[j].edges[2].e-1].z*((vertices[facets[j].edges[0].e-1].y*vertices[facets[j].edges[1].e-1].x)-(vertices[facets[j].edges[0].e-1].x*vertices[facets[j].edges[1].e-1].y)))) >= 0. ) {
             for(i = 1; i < (facets[j].count-1)/2+1; i++) {
                 int tmp;
                 tmp = facets[j].edges[i].e;
                 facets[j].edges[i].e = facets[j].edges[facets[j].count-i].e;
                 facets[j].edges[facets[j].count-i].e = tmp;
             }
         }
    }

    fprintf( stdout, "%d %d %d\n", nvertices, nfacets, nedges );
    for(i = 0; i < nvertices; i++) {
        fprintf( stdout, "%f %f %f\n", vertices[i].x, vertices[i].y,
                                       vertices[i].z );
    }
    for(j = 0; j < nfacets; j++) {
        printf( "%d ", facets[j].count );
        for(i = 0; i < facets[j].count; i++)
            printf( "%d ", facets[j].edges[i].e );
        printf( "\n" );
    }
}
